@@ -1,5 +1,9 @@
 .onAttach <- function(libname, pkgname){
-  initmessage1 <- "\n\n><<<<<<<<<<<<<<<<<<<<<<<<   EstimationTools Version "
+  invisible(suppressPackageStartupMessages(
+    sapply(c("GA", "DEoptim", "BBmisc", "survival"),
+           requireNamespace, quietly = TRUE)
+  ))
+  initmessage1 <- "\n><<<<<<<<<<<<<<<<<<<<<<<<   EstimationTools Version "
   initmessage2 <- utils::packageDescription("EstimationTools")$Version
   initmessage3 <- "   >>>>>>>>>>>>>>>>>>>>>>>><
   Feel free to report bugs in https://github.com/Jaimemosg/EstimationTools/issues"
